#include <mega32.h>
#include <delay.h>
#include "enc28j60.h"
#include "../uart/uart.h"

/*#define enc_start() ENC_CSPORT &= ~ENC_CSPIN
#define enc_stop() ENC_CSPORT |= ENC_CSPIN*/

void enc_start() {
  ENC_CSPORT = (0<<ENC_CSPIN);
}

void enc_stop() {
  ENC_CSPORT = (1<<ENC_CSPIN);
}

void enc_reset(void){
  enc_start();

  SPDR = ENCMD_SRC;
  while(!(SPSR & (1 << SPIF)));
 
  enc_stop();
}

void enc_bfs(unsigned char reg, unsigned char data){
  enc_start();

  SPDR = ENCMD_BFS|reg;
  while(!(SPSR & (1 << SPIF)));
 
  SPDR = data;
  while(!(SPSR & (1 << SPIF)));
 
  enc_stop();
}

void enc_bfc(unsigned char reg, unsigned char data){
  enc_start();

  SPDR = ENCMD_BFC|reg;
  while(!(SPSR & (1 << SPIF)));
 
  SPDR = data;
  while(!(SPSR & (1 << SPIF)));
 
  enc_stop();
}

void enc_wcr(unsigned char reg, unsigned char data){
  enc_start();
  
  SPDR = ENCMD_WCR|reg;
  while(!(SPSR & (1 << SPIF)));
  
  SPDR = data;
  while(!(SPSR & (1 << SPIF)));
  
  enc_stop();
}

unsigned char enc_rcr(unsigned char reg, unsigned char method){
  enc_start();
  
  ausgabe_uart_single_char(ENCMD_RCR);
  ausgabe_uart_single_char(reg);    
  ausgabe_uart_single_char(ENCMD_RCR|reg);
    
  SPDR = ENCMD_RCR|reg;
  delay_ms(2);  
  while(!(SPSR & (1 << SPIF)));
  delay_ms(2);
  /*SPDR = 0;
  while(!(SPSR & (1 << SPIF)));*/                  
  
  if(method) { // zum Lesen von PHY-Registern
    SPDR = 0;
    while(!(SPSR & (1 << SPIF)));
  }

  enc_stop();
  delay_ms(2);
    
  return(SPDR);
}

void enc_wbm(unsigned char *data, int len){
  int i;

  enc_start();

  SPDR = ENCMD_WBM;
  while(!(SPSR & (1 << SPIF)));
 
  for(i=0; i < len ;i++) { 
    SPDR = *data++;
    while(!(SPSR & (1 << SPIF)));
  }
  
  enc_stop();
}

void enc_rbm(unsigned char *data, int len){
  int i;

  enc_start();

  SPDR = ENCMD_RBM;
  while(!(SPSR & (1 << SPIF)));
 
  for(i=0; i < len ;i++) {
    SPDR = 0; while(!(SPSR & (1 << SPIF)));
    *data++ = SPDR;
  }
  
  enc_stop();
}

void enc_banksel(unsigned char bank){
  static unsigned char lastbank=0xf;

    enc_bfc(ECON1,ECON1_BSEL0|ECON1_BSEL1);
    enc_bfs(ECON1,bank);
    lastbank = bank;
}